#-----------------------------------------------------------------------
# File    : jnicoco.mak
# Contents: build JNICoCo dynamic link library (on Windows systems)
# Author  : Christian Borgelt
# History : 2013.12.10 file created
#           2015.03.04 interrupt signal handling added
#           2015.03.05 preprocessor definitions XXX_ABORT added 
#-----------------------------------------------------------------------
!IFNDEF JAVADIR
JAVADIR  = "C:\Program Files\Java\jdk-19"
!ENDIF
THISDIR  = ..\..\jnicoco\src
UTILDIR  = ..\..\util\src
MATHDIR  = ..\..\math\src
TRACTDIR = ..\..\tract\src
COCODIR  = ..\..\coconad\src

CC       = cl.exe
DEFS     = /D WIN32 /D NDEBUG /D _CONSOLE /D _CRT_SECURE_NO_WARNINGS
CFLAGS   = /nologo /MD /W3 /O2 /GS- $(DEFS) /c
JAVAINC  = /I $(JAVADIR)\include /I $(JAVADIR)\include\win32
INCS     = /I $(UTILDIR) /I$(MATHDIR) /I $(TRACTDIR) /I $(COCODIR)

LD       = link.exe
LDFLAGS  = /DLL /nologo /incremental:no
LIBS     = /LIBPATH:$(JAVADIR)\lib

HDRS     = $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h    \
           $(UTILDIR)\memsys.h   $(UTILDIR)\symtab.h    \
           $(UTILDIR)\sigint.h   $(TRACTDIR)\tract.h    \
           $(TRACTDIR)\train.h   $(TRACTDIR)\patspec.h  \
           $(TRACTDIR)\clomax.h  $(TRACTDIR)\cm4seqs.h  \
           $(TRACTDIR)\cmfilter.h $(TRACTDIR)\report.h
OBJS     = arrays.obj memsys.obj idmap.obj random.obj      \
           sigint.obj gamma.obj tract.obj train.obj        \
           patspec.obj cm4seqs.obj cmfilter.obj clomax.obj \
           report.obj coconad.obj ccnpsp.obj JNICoCo.obj $(ADDOBJS)
PRGS     = JNICoCo.dll

#-----------------------------------------------------------------------
# Build Dynamic Link Library
#-----------------------------------------------------------------------
all:          $(PRGS)

JNICoCo.dll:  $(OBJS) jnicoco.mak
	$(LD) $(LDFLAGS) $(LIBS) $(OBJS) /out:$@ /IMPLIB:JNICoCo.lib

#-----------------------------------------------------------------------
# Array Operations
#-----------------------------------------------------------------------
arrays.obj:   $(UTILDIR)\fntypes.h
arrays.obj:   $(UTILDIR)\arrays.h   $(UTILDIR)\arrays.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\arrays.c /Fo$@

#-----------------------------------------------------------------------
# Memory Management System for Objects of Equal Size
#-----------------------------------------------------------------------
memsys.obj:   $(UTILDIR)\memsys.h   $(UTILDIR)\memsys.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\memsys.c /Fo$@

#-----------------------------------------------------------------------
# Symbol Table Management
#-----------------------------------------------------------------------
idmap.obj:    $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h
idmap.obj:    $(UTILDIR)\symtab.h   $(UTILDIR)\symtab.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) /D IDMAPFN $(UTILDIR)\symtab.c /Fo$@

#-----------------------------------------------------------------------
# Random Number Generator Management
#-----------------------------------------------------------------------
random.obj:   $(UTILDIR)\random.h   $(UTILDIR)\random.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\random.c /Fo$@

#-----------------------------------------------------------------------
# Interrupt Signal Handling
#-----------------------------------------------------------------------
sigint.obj:   $(UTILDIR)\sigint.h   $(UTILDIR)\sigint.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(UTILDIR)\sigint.c /Fo$@

#-----------------------------------------------------------------------
# Gamma Function
#-----------------------------------------------------------------------
gamma.obj:    $(MATHDIR)\gamma.h    $(MATHDIR)\gamma.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(MATHDIR)\gamma.c /Fo$@

#-----------------------------------------------------------------------
# Item and Transaction Management
#-----------------------------------------------------------------------
tract.obj:    $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h \
              $(UTILDIR)\symtab.h
tract.obj:    $(TRACTDIR)\tract.h   $(TRACTDIR)\tract.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\tract.c /Fo$@

#-----------------------------------------------------------------------
# Train/Point Process Management
#-----------------------------------------------------------------------
train.obj:    $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h \
              $(UTILDIR)\symtab.h   $(TRACTDIR)\tract.h
train.obj:    $(TRACTDIR)\train.h   $(TRACTDIR)\train.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) /D TRN_SURR $(TRACTDIR)\train.c /Fo$@

#-----------------------------------------------------------------------
# Item Set Reporter Management
#-----------------------------------------------------------------------
patspec.obj:  $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h   \
              $(UTILDIR)\symtab.h   $(UTILDIR)\tabwrite.h \
              $(TRACTDIR)\tract.h
patspec.obj:  $(TRACTDIR)\patspec.h $(TRACTDIR)\patspec.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) /D PSP_ESTIM /D PSP_TRAIN \
	      $(TRACTDIR)\patspec.c /Fo$@

clomax.obj:   $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h \
              $(UTILDIR)\memsys.h   $(UTILDIR)\symtab.h \
              $(TRACTDIR)\tract.h
clomax.obj:   $(TRACTDIR)\clomax.h  $(TRACTDIR)\clomax.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\clomax.c /Fo$@

report.obj:   $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h   \
              $(UTILDIR)\symtab.h   $(UTILDIR)\scanner.h  \
              $(TRACTDIR)\tract.h   $(TRACTDIR)\patspec.h \
              $(TRACTDIR)\clomax.h
report.obj:   $(TRACTDIR)\report.h  $(TRACTDIR)\report.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) /D ISR_PATSPEC /D ISR_CLOMAX \
          /D ISR_NONAMES $(TRACTDIR)\report.c /Fo$@

#-----------------------------------------------------------------------
# Prefix Tree Management for Closed and Maximal Sequences
#-----------------------------------------------------------------------
cm4seqs.obj:  $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h \
              $(UTILDIR)\memsys.h   $(UTILDIR)\symtab.h \
              $(TRACTDIR)\tract.h   $(TRACTDIR)\report.h
cm4seqs.obj:  $(TRACTDIR)\cm4seqs.h $(TRACTDIR)\cm4seqs.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\cm4seqs.c /Fo$@

#-----------------------------------------------------------------------
# List/Array Closed/Maximal Filter Management
#-----------------------------------------------------------------------
cmfilter.obj: $(UTILDIR)\fntypes.h   $(UTILDIR)\arrays.h \
              $(UTILDIR)\symtab.h    $(TRACTDIR)\tract.h \
              $(TRACTDIR)\report.h
cmfilter.obj: $(TRACTDIR)\cmfilter.h $(TRACTDIR)\cmfilter.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) $(TRACTDIR)\cmfilter.c /Fo$@

#-----------------------------------------------------------------------
# CoCoNAD
#-----------------------------------------------------------------------
coconad.obj:  $(HDRS)               $(COCODIR)\coconad.h
coconad.obj:  $(COCODIR)\coconad.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT $(COCODIR)\coconad.c /Fo$@

#-----------------------------------------------------------------------
# Pattern Spectrum Functions
#-----------------------------------------------------------------------
ccnpsp.obj:   $(HDRS)               $(UTILDIR)\random.h \
              $(UTILDIR)\sigint.h   $(COCODIR)\coconad.h
ccnpsp.obj:   $(COCODIR)\ccnpsp.h   $(COCODIR)\ccnpsp.c jnicoco.mak
	$(CC) $(CFLAGS) $(INCS) /D CCN_ABORT $(COCODIR)\ccnpsp.c /Fo$@

#-----------------------------------------------------------------------
# Java Stuff
#-----------------------------------------------------------------------
JNICoCo.obj:  $(HDRS)               $(UTILDIR)\random.h  \
              $(UTILDIR)\sigint.h   $(TRACTDIR)\patred.h \
              $(COCODIR)\coconad.h  $(COCODIR)\ccnpsp.h  \
              JNICoCo.h JNICoCo.c
JNICoCo.obj:  JNICoCo.c jnicoco.mak
	$(CC) $(CFLAGS) $(JAVAINC) $(INCS) JNICoCo.c /Fo$@

#-----------------------------------------------------------------------
# Install
#-----------------------------------------------------------------------
install:
	-@copy $(PRGS) ..\..\..\bin

#-----------------------------------------------------------------------
# Clean up
#-----------------------------------------------------------------------
clean:
	-@erase /Q *~ *.obj *.idb *.pch *.exp *.lib $(PRGS)
